/*<!--
  **** BEGIN LICENSE BLOCK ****
  Version: MPL 1.1/GPL 2.0/LGPL 2.1
  Copyright(c) 2007-2010 Created by Alex Aster (www.alrond.com)
  for WorldIP-Project (www.WIPmania.com)
  All Rights Reserved.

  This program are subject to the Mozilla Public License Version
  1.1 (the "License"); you may not use this file except in compliance with
  the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/

  This program is free software. Software distributed under the License
  is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND,
  either express or implied. See the License for the specific language
  governing rights and limitations under the License.

  Alternatively, the contents of this file may be used under the terms of
  either the GNU General Public License Version 2 or later (the "GPL"), or
  the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  in which case the provisions of the GPL or the LGPL are applicable instead
  of those above. If you wish to allow use of your version of this file only
  under the terms of either the GPL or the LGPL, and not to allow others to
  use your version of this file under the terms of the MPL, indicate your
  decision by deleting the provisions above and replace them with the notice
  and other provisions required by the GPL or the LGPL. If you do not delete
  the provisions above, a recipient may use your version of this file under
  the terms of any one of the MPL, the GPL or the LGPL.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA 02111-1307 USA
  **** END LICENSE BLOCK ****
-->*/

const WIPCc = Components.classes;
const WIPCi = Components.interfaces;
const WIP_STATE_START = WIPCi.nsIWebProgressListener.STATE_START;
const WIP_TYPE_ONE_SHOT = WIPCi.nsITimer.TYPE_ONE_SHOT;
const WIP_TYPE_REPEATING_SLACK = WIPCi.nsITimer.TYPE_REPEATING_SLACK;
const WIP_TYPE_REPEATING_PRECISE = WIPCi.nsITimer.TYPE_REPEATING_PRECISE;
const WIP_VERSION = "2.2.1";
const WIP_UniqPrefix = "WorldIP";
const WIP_PLATFORM = WIPCc["@mozilla.org/xre/app-info;1"].getService(WIPCi.nsIXULAppInfo).platformVersion;
const WIP_OS = WIPCc["@mozilla.org/xre/app-info;1"].getService(WIPCi.nsIXULRuntime).OS;

window.addEventListener("load", function() { wip.init(); }, false);
window.addEventListener("unload", function() { wip.killer(); }, false);

var wipcookieremover =
{
	observe : function(subject, topic, data) {
		if (topic != "http-on-modify-request")
			return;
		var channel = subject.QueryInterface( WIPCi.nsIHttpChannel );
		if ( /api[123456]?\.wipmania\.com/.test( channel.originalURI.host ) ) {
			channel.setRequestHeader("Cookie", "", false);
		}
	}
}

var wip_storage =
{
	_cache: {},
	_timeout: {},
	_fuel: false,
	_UniqPrefix: "WIP",

	get: function(key) {
		key = String(key);
		if (key in this._cache) {
			if (this._timeout[key] && this.current_seconds() >= this._timeout[key] ) {
					delete this._cache[key];
					delete this._timeout[key];
			} else
				return this._cache[key];
		}
		if ( this._fuel )
			return this.get_global(key);
		return null;
	},

	get_global: function(key) {
		key = String(key);
		if ( this._fuel && this._fuel.storage.has(this._UniqPrefix+key) ) {
			var r = this._fuel.storage.get(this._UniqPrefix+key, null)
			var t = this._fuel.storage.get(this._UniqPrefix+"#t#"+key, 0);
			if (t && this.current_seconds() >= t) {
				this.delete(key);
				return null;
			}
			if ( !( key in this._cache && this._cache[key] == r ) ) {
				this._cache[key] = r;
				this._timeout[key] = t;
			}
			return r;
		}
		return null;
	},

	set: function(key, value, timeout) {
		key = String(key);
		if (typeof(timeout)!="number" || !timeout)
			timeout = 0; //forever
		else
			timeout = timeout + this.current_seconds();
		if ( this._fuel ) {
			this._fuel.storage.set(this._UniqPrefix+key, value);
			if (timeout)
				this._fuel.storage.set(this._UniqPrefix+"#t#"+key, timeout);
			else if (this._fuel.storage.has(this._UniqPrefix+"#t#"+key))
				this._fuel.storage.set(this._UniqPrefix+"#t#"+key, null)
		}
		this._cache[key] = value;
		this._timeout[key] = timeout;
	},

	delete: function(key) {
		key = String(key);
		if (key in this._cache) {
			delete this._cache[key];
			delete this._timeout[key];
		}
		if ( this._fuel && this._fuel.storage.has(this._UniqPrefix+key) ) {
			this._fuel.storage.set(this._UniqPrefix+key,null)
			if ( this._fuel.storage.has(this._UniqPrefix+"#t#"+key) )
				this._fuel.storage.set(this._UniqPrefix+"#t#"+key, null)
		}
	},

	current_seconds: function() {
		var lastTime = new Date().getTime();
		return Math.round(lastTime/1000);
	},
}

var wip =
{
	JSON: null,
	APIServerList: ["api1","api2","api3","api4","api5","api6"], //DE,US,JP,CL,AU,SG
	prefs: {},

	init: function()
	{
		var content = document.getElementById("content");
		wip.lock = 0;
		wip.currentUri = "";
		wip.longLastRequests = 0;
		wip.longLastRequestsDate = 0;
		wip.currentIP = "";
		wip.proxy_dns_enabled = false;

		// cache between firefox-windows
		try {
			var WIP_BrowserName = document.getElementById("bundle_brand").getString("brandShortName");
			if ( WIP_BrowserName == "SeaMonkey" )
				wip_storage._fuel = WIPCc["@mozilla.org/smile/application;1"].getService(WIPCi.smileIApplication);
			else if ( WIP_BrowserName == "Mozilla" || WIP_BrowserName == "Firefox" )
				wip_storage._fuel = WIPCc["@mozilla.org/fuel/application;1"].getService(WIPCi.fuelIApplication);
		} catch(e) {}

		// check FF version and make better JSON parser
		// originally by Myk Melez <myk@mozilla.org> from JSON.js
		//var appInfo = WIPCc["@mozilla.org/xre/app-info;1"].getService(WIPCi.nsIXULAppInfo);
		//if (appInfo.platformVersion.indexOf("1.9.0") == 0 || appInfo.platformVersion == "1.9") {
		if (WIP_PLATFORM.indexOf("1.9.0") == 0 || WIP_PLATFORM == "1.9") {
			var nativeJSON = WIPCc["@mozilla.org/dom/json;1"].createInstance(WIPCi.nsIJSON);
			wip.JSON = {
				nativeJSON: nativeJSON,
				parse: function(jsonString) { return this.nativeJSON.decode(jsonString) },
				stringify: function(jsObject) { return this.nativeJSON.encode(jsObject) }
			};
		} else wip.JSON = JSON;
		wip.worldip_myip_leer = document.getElementById("worldip-myip").getAttribute("label");
		wip.worldip_rdns_leer = document.getElementById("worldip-getrdns").getAttribute("label");
		wip.worldip_ip_leer = document.getElementById("worldip-ip").getAttribute("label");
		var prefService = WIPCc["@mozilla.org/preferences-service;1"];
		wip.preferences = prefService.getService(WIPCi.nsIPrefService).getBranch("extensions.worldip.");
		wip.preferences.QueryInterface(WIPCi.nsIPrefBranch2);
		wip.preferencesProxy = prefService.getService(WIPCi.nsIPrefService).getBranch("network.proxy.");
		wip.preferencesProxy.QueryInterface(WIPCi.nsIPrefBranch2);
		wip.currentAPI = "api";
		wip.observerService = WIPCc["@mozilla.org/observer-service;1"].getService(WIPCi.nsIObserverService);
		wip.initPrefs();
		wip.cookieDisable();
		if ( wip_storage.get("update_version_lock") != true ) {
			wip_storage.set("update_version_lock", true, 15);
			wip.updateVersion();
		}
		wip.updateMenuItems();
		wip.generateMenu();
		wip.preferences.addObserver("", this, false);
		wip.preferencesProxy.addObserver("", this, false);
		wip.setPositon();
		
		wip.MyIPTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
		if (wip.prefs["enablemyip"])
			wip_storage.get_global("myip");
		else if (wip.prefs["enablelands"])
			wip.getMyIP();
		else
			wip.setPanelAttributes();
		wip.showInfo(null,null);

		this.Listener =
		{
			QueryInterface: function(aIID)
			{
				if (aIID.equals(WIPCi.nsIWebProgressListener) ||
				aIID.equals(WIPCi.nsISupportsWeakReference) ||
				aIID.equals(WIPCi.nsIXULBrowserWindow) ||
				aIID.equals(WIPCi.nsISupports))
				return this;
				throw Components.results.NS_NOINTERFACE;
			},
			onLocationChange:function(a,b,c)
			{
				try {
					wip.onPageLoad(c);
				} catch(e) {}
			},
			onProgressChange:function(a,b,c,d,e,f){},
			onSecurityChange:function(a,b,c){},
			onStateChange:function(aProgress,aRequest,aFlag,aStatus) {},
			onStatusChange:function(a,b,c,d){},
			onLinkIconAvailable:function(a){}
		};
		try {
			content.addProgressListener(this.Listener); //, WIP_STATE_START
			document.getElementById("home-button").addEventListener("click", wip.onTabSelect, false);
			window.getBrowser().tabContainer.addEventListener("TabOpen", wip.onTabOpen, false);
			window.getBrowser().addEventListener("select", wip.onTabSelect, true);
		} catch(e) {}
	},

	killer: function()
	{
		var content = document.getElementById("content");
		content.removeProgressListener(this.Listener);
		window.getBrowser().tabContainer.removeEventListener("TabOpen", wip.onTabOpen, false);
		window.getBrowser().removeEventListener("select", wip.onTabSelect, true);
		wip.preferences.removeObserver("", this);
		wip.preferencesProxy.removeObserver("", this);
		wip.cookieDisableOff();
	},

	setTimer: function(callbackFunc, delay, typ)
	{
		var wipTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
		var event = {
			notify: function(timer) {
				try {
					callbackFunc.call();
				} catch(e) {}
			}
		};
		try {
			wipTimer.initWithCallback(event,delay,typ);
		} catch(e) {}
	},

	initPrefs: function()
	{
		wip.prefs["myiplogenable"] = false;
		wip.proxy_dns_enabled = false;
		wip.prefs["proxy_dns_uselocal"] = wip.preferences.getBoolPref("proxy_dns_uselocal");
		wip.prefs["proxy_type"]= wip.preferencesProxy.getIntPref("type");
		wip.prefs["proxy_socks"]= wip.preferencesProxy.getCharPref("socks");
		wip.prefs["proxy_socks_port"]= wip.preferencesProxy.getIntPref("socks_port");
		wip.prefs["proxy_socks_version"]= wip.preferencesProxy.getIntPref("socks_version");
		wip.prefs["proxy_socks_remote_dns"]= wip.preferencesProxy.getBoolPref("socks_remote_dns");
		if ( wip.prefs["proxy_type"] == 1 )
			wip.proxy_dns_enabled = (wip.prefs["proxy_socks"]!="") && (wip.prefs["proxy_socks_port"]>0) && (wip.prefs["proxy_socks_version"]==5) && wip.prefs["proxy_socks_remote_dns"];
		wip.prefs["enablemyip"]= wip.preferences.getBoolPref("enablemyip");
		wip.prefs["updatemyiptimer"]= wip.preferences.getCharPref("updatemyiptimer");
		if (wip.prefs["updatemyiptimer"]=="m") // minutes
			wip.prefs["updatemyipinterval"]= 60 * wip.preferences.getIntPref("updatemyipinterval");
		else // hours
			wip.prefs["updatemyipinterval"]= 3600 * wip.preferences.getIntPref("updatemyipinterval");
		wip.prefs["position"]= wip.preferences.getCharPref("position"); //s,a,t - statusbar,addressbar,toolbar
		wip.prefs["toolbarposition"]= wip.preferences.getCharPref("toolbarposition"); //left,center,right
		wip.prefs["clickaction"]= wip.preferences.getCharPref("clickaction"); //no,land,copy,newip,wrongland,getland,rdns,dns,asonhe
		wip.prefs["myiplogenable"]= wip.preferences.getBoolPref("myiplogenable");
		wip.prefs["showflag"]= wip.preferences.getBoolPref("showflag");
		wip.prefs["flagset"]= wip.preferences.getCharPref("flagset");
		wip.prefs["showland"]= wip.preferences.getBoolPref("showland");
		wip.prefs["showlandcode"]= wip.preferences.getBoolPref("showlandcode");
		wip.prefs["showcity"]= wip.preferences.getBoolPref("showcity");
		wip.prefs["showcitycolor"]= wip.preferences.getCharPref("showcitycolor");
		wip.prefs["showip"]= wip.preferences.getBoolPref("showip");
		wip.prefs["showmyip"]= wip.preferences.getBoolPref("showmyip");
		wip.prefs["showlandcolor"]= wip.preferences.getCharPref("showlandcolor");
		wip.prefs["showlandcodecolor"]= wip.preferences.getCharPref("showlandcodecolor");
		wip.prefs["showipcolor"]= wip.preferences.getCharPref("showipcolor");
		wip.prefs["showmyipcolor"]= wip.preferences.getCharPref("showmyipcolor");
		wip.prefs["enableflagcache"]= wip.preferences.getBoolPref("enableflagcache");
		wip.prefs["enablelands"]= wip.preferences.getBoolPref("enablelands");
		wip.prefs["lastmyland"]= wip.preferences.getCharPref("lastmyland");
		wip.prefs["showdatacenter"]= wip.preferences.getBoolPref("showdatacenter");
		wip.prefs["showdatacentercolor"]= wip.preferences.getCharPref("showdatacentercolor");
		wip.prefs["showasnumber"]= wip.preferences.getBoolPref("showasnumber");
		wip.prefs["showasnumbercolor"]= wip.preferences.getCharPref("showasnumbercolor");
		wip.prefs["showasname"]= wip.preferences.getBoolPref("showasname");
		wip.prefs["showasnamecolor"]= wip.preferences.getCharPref("showasnamecolor");
		wip.prefs["showmyasname"]= wip.preferences.getBoolPref("showmyasname");
		wip.prefs["showmyasnamecolor"]= wip.preferences.getCharPref("showmyasnamecolor");
		wip.prefs["showmyasnumber"]= wip.preferences.getBoolPref("showmyasnumber");
		wip.prefs["showmyasnumbercolor"]= wip.preferences.getCharPref("showmyasnumbercolor");
		wip.prefs["showmydatacenter"]= wip.preferences.getBoolPref("showmydatacenter");
		wip.prefs["showmydatacentercolor"]= wip.preferences.getCharPref("showmydatacentercolor");
		wip.prefs["showrdns"]= wip.preferences.getBoolPref("showrdns");
		wip.prefs["showrdnscolor"]= wip.preferences.getCharPref("showrdnscolor");
		wip.prefs["showmyrdns"]= wip.preferences.getBoolPref("showmyrdns");
		wip.prefs["showmyrdnscolor"]= wip.preferences.getCharPref("showmyrdnscolor");
		wip.prefs["acceptinrank"]= wip.preferences.getBoolPref("acceptinrank");
		wip.prefs["api1ResponseTime"]= wip.preferences.getIntPref("api1ResponseTime");
		wip.prefs["api2ResponseTime"]= wip.preferences.getIntPref("api2ResponseTime");
		wip.prefs["api3ResponseTime"]= wip.preferences.getIntPref("api3ResponseTime");
		wip.prefs["api4ResponseTime"]= wip.preferences.getIntPref("api4ResponseTime");
		wip.prefs["api5ResponseTime"]= wip.preferences.getIntPref("api5ResponseTime");
		wip.prefs["api6ResponseTime"]= wip.preferences.getIntPref("api6ResponseTime");
		wip.prefs["APIServerListMD5"]= wip.preferences.getCharPref("APIServerListMD5");
		wip.prefs["APIServerFolge"]= wip.preferences.getCharPref("APIServerFolge");
		wip.prefs["APIServer"]= wip.prefs["APIServerFolge"] ? wip.prefs["APIServerFolge"][0] : "";
		wip.currentAPI = wip.prefs["APIServer"] ? wip.APIServerList[wip.prefs["APIServer"]] : "api";
		wip.prefs["nextServerTime"]= (wip.prefs["APIServerFolge"].length > 1) ? wip.prefs[wip.APIServerList[wip.prefs["APIServerFolge"][1]]+"ResponseTime"] : 0;
		wip.prefs["thisServerTime"]= (wip.prefs["APIServerFolge"].length > 0) ? wip.prefs[wip.APIServerList[wip.prefs["APIServerFolge"][0]]+"ResponseTime"] : 0;
	},

	updateVersion: function()
	{
		wip.prefs["version"] = wip.preferences.getCharPref("version");
		if (!wip.prefs["version"] || (wip.prefs["version"] != WIP_VERSION))
		{
			try { wip.preferences.clearUserPref("enablehttpserver"); } catch(e) {}
			try { wip.preferences.clearUserPref("apiResponseTime"); } catch(e) {}
			wip.preferences.setCharPref("version",WIP_VERSION);
			var lastTime = new Date().getTime();
			var lastTimeDays = Math.round(lastTime/(24*3600*1000)); //days from Epoch
			//wip.preferences.setIntPref("pointstime", lastTimeDays-5); // make first check only tomorrow or later
			wip.preferences.setIntPref("pointstime", 1); // make first check only tomorrow or later
			var host = "www.wipmania.com";
			var ip = wip.ipResolve(host);
			if (ip)
				wip.getLand(ip,host,0);
			wip.setTimer(function() { wip.goHome("plugins/"); }, 500, WIP_TYPE_ONE_SHOT);
			wip.updateAPIFolge();
		}
		if (!wip.prefs["APIServerListMD5"] || !wip.prefs["APIServerFolge"] || wip.prefs["APIServerFolge"].length < 3 )
			wip.updateAPIFolge();
		return true;
	},

	updateAPIFolge: function()
	{
		var req = new XMLHttpRequest();
		req.open("GET", "http://api.wipmania.com/md5api.txt", true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var doc = req.responseText;
				var md5api = doc.substring(0,33);
				if (md5api != wip.prefs["APIServerListMD5"] || !wip.prefs["APIServerFolge"] || wip.prefs["APIServerFolge"].length < 3 || wip.currentAPI=="api")
				{
					wip.prefs["APIServerListMD5"] = md5api;
					wip.preferences.setCharPref("APIServerListMD5", md5api);
					for (Server in wip.APIServerList)
					{
						wip.preferences.setIntPref(wip.APIServerList[Server]+"ResponseTime", 0);
						try {
							var ip = wip.ipResolve(wip.APIServerList[Server]+".wipmania.com");
							if (ip || wip.proxy_dns_enabled)
								wip.getAPIResponseTime(wip.APIServerList[Server],0);
						} catch(e) {}
					}
					wip.setTimer(function() { wip.orderAPIbyResponseTime(); }, 5000, WIP_TYPE_ONE_SHOT);
				}
			}
		};
		req.send(null);
	},

	orderAPIbyResponseTime: function()
	{
		var times = [];
		var timesServer = {};
		var folge = "";
		for (Server in wip.APIServerList)
		{
			var thisResponseTime = wip.preferences.getIntPref(wip.APIServerList[Server]+"ResponseTime");
			if (!thisResponseTime || thisResponseTime<0)
				wip.preferences.setIntPref(wip.APIServerList[Server]+"ResponseTime", -1);
			else
			{
				times.push(parseInt(thisResponseTime));
				timesServer[parseInt(thisResponseTime)]=Server;
			}
		}
		times.sort(function(a,b){return a - b});
		for (t in times) { folge += timesServer[times[t]]; }
		wip.preferences.setCharPref("APIServerFolge", folge);
		wip_storage.delete("update_version_lock");
		wip.initPrefs();
		if (wip.prefs["enablelands"])
			wip.getMyIP();
	},

	getAPIResponseTime: function(Server,counter)
	{
		if (counter > 1) return;
		var req = new XMLHttpRequest();
		req.open("GET", "http://"+Server+".wipmania.com/", true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var endTime = new Date().getTime();
				var doc = req.responseText;
				if (doc.indexOf('<br>')>0)
				{
					var oldTime = wip.preferences.getIntPref(Server+"ResponseTime");
					if ( (oldTime > 0 && (endTime-startTime) < oldTime) || oldTime <= 0)
						wip.preferences.setIntPref(Server+"ResponseTime", endTime-startTime);
					wip.setTimer(function() { wip.getAPIResponseTime(Server,counter+1); }, 100, WIP_TYPE_ONE_SHOT);
				}
			}
		};
		var startTime = new Date().getTime();
		req.send(null);
	},

	updateMenuItems: function()
	{
		var currentTime = new Date().getTime();
		var currentTimeDays = Math.round(currentTime/(24*3600*1000));
		var lastTimeDays = wip.preferences.getIntPref("pointstime");
		wip.prefs["points"]= wip.JSON.parse(wip.preferences.getCharPref("points"));
		if ((currentTimeDays-lastTimeDays)>6) // check one time per week for new list of ping-servers
			wip.setTimer(function() { wip.getNewPoints(); }, 1000, WIP_TYPE_ONE_SHOT);
	},

	getNewPoints: function()
	{
		var req = new XMLHttpRequest();
		req.open("GET", "http://api.wipmania.com/points.txt", true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var lastTime = new Date().getTime();
				var lastTimeDays = Math.round(lastTime/(24*3600*1000)); //days from Epoch
				wip.preferences.setIntPref("pointstime", lastTimeDays);
				var doc = req.responseText;
				try {
					var newpoints = wip.JSON.parse(doc);
					if (!wip.prefs["points"].v || newpoints.v!=wip.prefs["points"].v){
						wip.preferences.setCharPref("points", doc);
						if (newpoints.m)
							wip.showMessage(newpoints.m,newpoints.mclick);
						wip.updateMenuItems();
						wip.generateMenu();
					}
				} catch(e) {}
			}
		};
		req.send(null);
	},

	generateSubMenu: function(act,command)
	{
		var points = wip.prefs["points"];
		var stringbundle = document.getElementById("worldip-tooltiplabels");
		var menupopup = document.createElement("menupopup");
		var menuitemworld = document.createElement("menuitem");
			menuitemworld.setAttribute("oncommand", "wip.goHome"+act+"("+command+",'world');");
			menuitemworld.setAttribute("label", stringbundle.getString("worldwide"));
			menuitemworld.setAttribute("class", "menuitem-iconic worldip-menuitem-iconic");
			menuitemworld.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/--.png);");
		menupopup.appendChild(menuitemworld);
		if (points.p) {
			for (point in points.p){
			if (points.p[point].subcontinent) {
						menupopup.appendChild(document.createElement("menuseparator"));
						var kontinent = document.createElement("menuitem");
						try { var kontinentname = stringbundle.getString(points.p[point].subcontinent); } catch(e) { var kontinentname = points.p[point].subcontinent; }
						kontinent.style.fontWeight = "bold";
						kontinent.setAttribute("label", kontinentname);
						menupopup.appendChild(kontinent);
			} else {
				var menuitemcity = document.createElement("menuitem");
					menuitemcity.setAttribute("oncommand", "wip.goHome"+act+"("+command+", '"+point+"');");
					try { var cityname = stringbundle.getString(point); } catch(e) { var cityname = point; }
					menuitemcity.setAttribute("label", cityname+", "+stringbundle.getString(points.p[point]));
					menuitemcity.setAttribute("class", "menuitem-iconic worldip-menuitem-iconic");
					menuitemcity.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/"+points.p[point]+".png);");
				menupopup.appendChild(menuitemcity);
			}
			}
		}
		return menupopup;
	},

	generateMenu: function()
	{
		var ping_ip_menu = document.getElementById("worldip-ping-ip-menu");
		var trace_ip_menu = document.getElementById("worldip-trace-ip-menu");
		var ping_myip_menu = document.getElementById("worldip-ping-myip-menu");
		var trace_myip_menu = document.getElementById("worldip-trace-myip-menu");
		var provider_byname = document.getElementById("worldip-provider-byname");
		var provider_bycity = document.getElementById("worldip-provider-bycity");
		var stringbundle = document.getElementById("worldip-tooltiplabels");
		var points = wip.prefs["points"];

		while (ping_ip_menu.lastChild)
			ping_ip_menu.removeChild(ping_ip_menu.lastChild);
		while (trace_ip_menu.lastChild)
			trace_ip_menu.removeChild(trace_ip_menu.lastChild);
		while (ping_myip_menu.lastChild)
			ping_myip_menu.removeChild(ping_myip_menu.lastChild);
		while (trace_myip_menu.lastChild)
			trace_myip_menu.removeChild(trace_myip_menu.lastChild);
		while (provider_byname.lastChild)
			provider_byname.removeChild(provider_byname.lastChild);
		while (provider_bycity.lastChild)
			provider_bycity.removeChild(provider_bycity.lastChild);

		var submenu = wip.generateSubMenu("Ping","wip.getCurrentIP()");
		ping_ip_menu.appendChild(submenu);
		var submenu = wip.generateSubMenu("Trace","wip.getCurrentIP()");
		trace_ip_menu.appendChild(submenu);
		var submenu = wip.generateSubMenu("Ping","wip.myip()");
		ping_myip_menu.appendChild(submenu);
		var submenu = wip.generateSubMenu("Trace","wip.myip()");
		trace_myip_menu.appendChild(submenu);

		var menuitemproviderbynamepopup = document.createElement("menupopup");
			if (points.pr) {
				for (prov in points.pr){
					var provitem = document.createElement("menuitem");
						provitem.setAttribute("label", points.pr[prov].c);
						provitem.setAttribute("class", "menuitem-iconic worldip-menuitem-iconic");
						provitem.setAttribute("style", "list-style-image: url(chrome://worldip/skin/prov/"+prov+".png);");
						if (points.pr[prov].la) // qwest hack
							provitem.setAttribute("oncommand", "wip.goProvider('"+points.pr[prov].l+"');wip.goProvider('"+points.pr[prov].la+"');");
						else
							provitem.setAttribute("oncommand", "wip.goProvider('"+points.pr[prov].l+"');");
					menuitemproviderbynamepopup.appendChild(provitem);
				}
			}
		provider_byname.appendChild(menuitemproviderbynamepopup);

		var menuitemproviderbycitypopup = document.createElement("menupopup");
			if (points.prc) {
				for (land in points.prc){
				if (points.prc[land].subcontinent) {
						menuitemproviderbycitypopup.appendChild(document.createElement("menuseparator"));
						var kontinent = document.createElement("menuitem");
						try { var kontinentname = stringbundle.getString(points.prc[land].subcontinent); } catch(e) { var kontinentname = points.prc[land].subcontinent; }
						kontinent.style.fontWeight = "bold";
						kontinent.setAttribute("label", kontinentname);
						menuitemproviderbycitypopup.appendChild(kontinent);
				} else {
					var landmenu = document.createElement("menu");
						landmenu.setAttribute("label", stringbundle.getString(land));
						landmenu.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/"+land+".png);");
						landmenu.setAttribute("class", "menu-iconic");
						var landpopup = document.createElement("menupopup");
						for (city in points.prc[land]) {
							var citymenu = document.createElement("menu");
								try { var cityname = stringbundle.getString(city); } catch(e) { var cityname = city; }
								citymenu.setAttribute("label", cityname);
								var citymenupopup = document.createElement("menupopup");
								for (prov in points.prc[land][city]) {
									var menuitemprov = document.createElement("menuitem");
										var url = points.pr[prov].l;
										if (points.pr[prov].la && land!="US") // qwest hack
											url = points.pr[prov].la;
										menuitemprov.setAttribute("oncommand", "wip.goProvider('"+url+"');");
										menuitemprov.setAttribute("label", points.pr[prov].c);
										menuitemprov.setAttribute("class", "menuitem-iconic worldip-menuitem-iconic");
										menuitemprov.setAttribute("style", "list-style-image: url(chrome://worldip/skin/prov/"+prov+".png);");
									citymenupopup.appendChild(menuitemprov);
								}
								citymenu.appendChild(citymenupopup);
							landpopup.appendChild(citymenu);
						}
						landmenu.appendChild(landpopup);
					menuitemproviderbycitypopup.appendChild(landmenu);
				}
				}
			}
		provider_bycity.appendChild(menuitemproviderbycitypopup);
	},

	cookieDisable: function() {
		wip.observerService.addObserver(wipcookieremover, "http-on-modify-request", false);
	},

	cookieDisableOff: function() {
		wip.observerService.removeObserver(wipcookieremover, "http-on-modify-request");
	},

	observe: function(subject, topic, data)
	{
		if (topic != "nsPref:changed")
			return;
		if (data=="APIServerListMD5" || data=="APIServerFolge" || data=="api1ResponseTime" || data=="api2ResponseTime" || data=="api3ResponseTime" || data=="api4ResponseTime"  || data=="api5ResponseTime"  || data=="api6ResponseTime" )
			return;
		if (data=="type" || data=="socks" || data=="socks_port" || data=="socks_version" || data=="socks_remote_dns" || data=="proxy_dns_uselocal")
		{
			wip.initPrefs();
			wip.changeMyIP();
			return;
		}
		if (data=="enablemyip")
			wip.enableMyIP(0);
		else if (data=="updatemyipinterval" || data=="updatemyiptimer")
		{
			wip.initPrefs();
			wip.changeMyIP();
		}
		else if (data=="flagset")
		{
			wip.prefs["flagset"] = wip.preferences.getCharPref("flagset");
			var panel_flag = document.getElementById("worldip-statusbar-flag");
			var src = panel_flag.getAttribute("src");
			if (wip.prefs["flagset"]=="0")
				src = src.replace("flags1","flags0");
			else
				src = src.replace("flags0","flags1");
			panel_flag.setAttribute("src", src);
			wip.setPanelAttributes();
			wip.generateMenu();
		}
		else if (data=="myiplogenable") {
			wip_storage.delete("myiplog");
			wip.initPrefs();
			wip.setPanelAttributes();
		}
		else
		{
			wip.initPrefs();
			wip.setPositon();
			wip.setPanelAttributes();
		}
	},

	onTabOpen: function(e)
	{
		var b = window.getBrowser().getBrowserForTab(e.target);
		wip.isHostReady(e.target,0);
	},

	isHostReady: function(target,time)
	{
		if (time > 6000)
			return false;
		var b = window.getBrowser().getBrowserForTab(target);
		try
		{
			var host = b.currentURI.host;
			var scheme = b.currentURI.scheme;
			if (!((scheme == "http")||(scheme == "https")||(scheme == "ftp")))
				return false;
			var ip = wip.ipResolve(host);
			if (ip)
				wip.getLand(ip,host,0);
		}
		catch(e)
		{
			var thisTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
			var event = { notify: function(timer) { wip.isHostReady(target,time+100); } };
			try { thisTimer.initWithCallback(event,100,WIP_TYPE_ONE_SHOT); } catch(e) {}
		}
	},

	onPageLoad: function(Location)
	{
		if (!wip.lock)
		{
			if (Location && (Location.scheme != "chrome") && (Location.scheme != "file") && (Location.scheme != "about") && Location.host )
			{
				var ip = wip.ipResolve(Location.host);
				wip.currentUri = Location;
				wip.currentIP = ip;
				if (!ip)
					wip.showInfo(null,Location.host,1);
				else
					wip.getLand(ip,Location.host,1);
			}
			else
			{
				try
				{
					var host = Location.host;
				} catch(e)
				{
					var host = null;
					if ( Location.scheme == "chrome" || Location.scheme == "file" )
						host = Location;
				}
				wip.showInfo(null,host);
				wip.currentUri = "";
				wip.currentIP = "";
			}
		}
		wip.lock = 0;
		return true;
	},

	onTabSelect: function(e)
	{
		var nodeName = e.originalTarget.nodeName;
		if (nodeName == "xul:tabs" || nodeName == "toolbarbutton")
		{
			wip.currentUri = "";
			wip.currentIP = "";
			if (nodeName == "toolbarbutton")
				wip.showInfo(null,null);
			return false;
		}
		var selectedTab = e.originalTarget.selectedIndex;
		if (!e.originalTarget.childNodes || !e.originalTarget.childNodes[selectedTab]) {
			wip.currentUri = "";
			wip.currentIP = "";
			wip.showInfo(null,null);
			return;
		}
		var selectedDoc = e.originalTarget.childNodes[selectedTab].firstChild;
		if (!selectedDoc || !selectedDoc.contentDocument) {
			wip.currentUri = "";
			wip.currentIP = "";
			wip.showInfo(null,null);
			return;
		}
		var uri = selectedDoc.contentDocument.location;
		wip.lock=1;
		try {
			var ip = wip.ipResolve(uri.host);
			if (ip) {
				wip.currentUri = uri;
				wip.currentIP = ip;
				wip.showInfo(ip,uri.host);
			} else {
				wip.currentUri = uri;
				wip.currentIP = "";
				wip.showInfo("",uri.host);
			}
		} catch(e)
		{
			wip.currentUri = "";
			wip.currentIP = "";
			wip.showInfo(null,null);
		}
		return true;
	},

	ipRemoteResolve: function()
	{
		host = wip.currentUri.host.split(':')[0];
		var d = wip_storage.get("dns"+host);
		if (d && d[0]) {
			if (d[0].indexOf(':')>0) {
				d.sort(function(a,b){return a.indexOf(":") >= b.indexOf(":")});
				return d[0];
			}
			else
				return d[0];
		}
		else if (d != null && (d === 0 || (d.length && (d[0] === 0 || d[0] === "")) ) )
			return null;

		var req = new XMLHttpRequest();
		req.open("GET", "https://www.wipmania.com/dns/?d="+host, true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var doc = req.responseText;
				var pattern = /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/;
				if (doc && !pattern.test(doc)) doc = 0;
				if (!doc && doc==="") doc = "";
				wip_storage.set("dns"+host, [doc]);
				wip.getLand(doc,host,1);
			}
		};
		req.send(null);
	},

	removeDuplicate: function(a)
	{
		var _temp = {};
		var temp = [];
		for(i=0; i<a.length; i++){
			if (!(a[i] in _temp)) {
				_temp[a[i]] = true;
				temp.push(a[i]);
			}
		}
		return temp;
	},

	ipResolve: function(fullhost) // on resolv return always IPv4, but in the sequence we have really list
	{
		host = fullhost.split(':')[0];
		if (host.length == 4) {
			if (host.indexOf('.')<0)
				host = fullhost; // IPv6
		}
		var d = wip_storage.get("dns"+host);
		if (d && d[0]) {
			if (d[0].indexOf(':')>0) {
				d.sort(function(a,b){return a.indexOf(":") >= b.indexOf(":")});
				return d[0];
			}
			else
				return d[0];
		}
		else if (d != null && (d === 0 || (d.length && (d[0] === 0 || d[0] === "")) ) )
			return null;

		if (wip.proxy_dns_enabled && !wip.prefs["proxy_dns_uselocal"])
			return null;

		if (host.indexOf(':')>0){
			wip_storage.set("dns"+host, [host], 3600);
			return host;
		}

		var dnsclass = WIPCc['@mozilla.org/network/dns-service;1'];
		var dns = dnsclass.getService(WIPCi.nsIDNSService);
		try {
			var resolved = dns.resolve(host, 0);
			if (!resolved) {
				wip_storage.set("dns"+host, 0, 3600);
				return null;
			}
			var ips = [];
			while (resolved.hasMore())
				ips.push(resolved.getNextAddrAsString());
			if (ips) {
				ips = wip.removeDuplicate(ips);
				var sorted_ips = ips;
				sorted_ips.sort(function(a,b){return a.indexOf(":") >= b.indexOf(":")});
				var ip = sorted_ips[0];
			} else {
				var ip = "";
			}

			var pattern = /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/;
			if (pattern.test(ip) && ip.indexOf(":")==-1)
			{
				wip_storage.set("dns"+host, ips, 3600);
				return ip;
			}
			else if ( ip == "::1" )
			{
				wip_storage.set("dns"+host, ["127.0.0.1", "::1"], 3600);
				return "127.0.0.1";
			}
			else // we doesn't work with IPv6
			{
				wip_storage.set("dns"+host, ips, 3600);
				return ip; // "V6"
			}
		} catch(e) {
			wip_storage.set("dns"+host, 0, 3600);
			return null;
		}
	},

	isPublicNetz: function(ip)
	{
		if (!ip)
			return false;
		var pattern = /^(0\.|10\.|127\.|192\.168|169\.254|198\.18\.|198\.19\.|192\.0\.0\.|192\.0\.2\.|198\.51\.100\.|203\.0\.113\.|192\.88\.99|2[3-5]\d|22[4-9]).*/; //RFC 3330,1918,2544,3171,1700,5736
		if (pattern.test(ip))
			return false;
		else if ((ip.indexOf("172.") == 0) && (16 <= parseInt(ip.split(".")[1])) && (parseInt(ip.split(".")[1]) <= 31)) // RFC 1918
			return false;
		else if ((ip.indexOf("100.") == 0) && (64 <= parseInt(ip.split(".")[1])) && (parseInt(ip.split(".")[1]) <= 127)) // RFC 6598
			return false;
		return true;
	},

	getIANAStatus: function(ip)
	{
		if (!ip)
			return false;
		return wipIPspace[ip.split(".")[0]];
	},

	getPrivateType: function(ip)
	{
		if (!ip)
			return false;
		if (ip.indexOf("127.") == 0)
			return "localhost";
		else if ((ip.indexOf("10.") == 0) || (ip.indexOf("192.168.") == 0))
			return "intnetwork";
		else if ((ip.indexOf("172.") == 0) && (16 <= parseInt(ip.split(".")[1])) && (parseInt(ip.split(".")[1]) <= 31))
			return "intnetwork";
		else if ((ip.indexOf("100.") == 0) && (64 <= parseInt(ip.split(".")[1])) && (parseInt(ip.split(".")[1]) <= 127))
			return "RFC6598";
		else if (ip.indexOf("0.") == 0)
			return "thisnetwork";
		else if (ip.indexOf("169.254.") == 0)
			return "linklocal";
		else if ((ip.indexOf("192.0.0.") == 0) || (ip.indexOf("192.0.2.") == 0) || (ip.indexOf("198.51.100.") == 0) || (ip.indexOf("203.0.113.") == 0))
			return "testnet";
		else if ((ip.indexOf("198.18.") == 0) || (ip.indexOf("198.19.") == 0))
			return "networktesting";
		else if (ip.indexOf("192.88.99") == 0)
			return "relay6to4";
		else if (ip == "V6" || ip.indexOf(":") >= 0)
			return "V6";
		else
			return false;
	},

	getDisabledLand: function()
	{
		var ip = wip.getCurrentIP();
		var host = wip.currentUri.host;
		wip.prefs["enablelands"] = true;
		wip.getLand(ip,host,1);
		wip.prefs["enablelands"] = false;
	},

	getLand: function(ip,host,barRenew)
	{
		var ipcache = wip_storage.get("ip"+ip);
		if (!wip_storage.get("iplock"+ip) && wip.isPublicNetz(ip) && ip!="V6" && ip.indexOf(":")<0 && (wip.prefs["enablelands"] || !host))
		{
			if (wip.prefs["enableflagcache"] && ipcache && ipcache.l && (host || ipcache.a) )
			{
				wip_storage.set("iplock"+ip, false);
				if (barRenew)
					wip.showInfo(ip,host);
				return true;
			}
			var req = new XMLHttpRequest();
			wip_storage.set("iplock"+ip, true, 20);
			req.open("GET", "http://"+wip.currentAPI+".wipmania.com/"+ip+"?"+host, true);
			req.setRequestHeader("Accept", "text/html,text/plain");
			req.setRequestHeader("Accept-Encoding", null);
			req.setRequestHeader("Keep-Alive", null);
			req.setRequestHeader("Accept-Charset", null);
			req.setRequestHeader("Connection", "close");
			if (wip.prefs["acceptinrank"])
				req.setRequestHeader("Parameter", "rank");
			req.setRequestHeader("Typ", "ffplugin2");
			req.setRequestHeader("Ver", WIP_VERSION);
			req.onreadystatechange = function()
			{
				if (req.readyState == 4)
				{
					var endTime = new Date().getTime();
					var reqTime = endTime-startTime;
					var doc = req.responseText;
					try {
						var apiresults = wip.JSON.parse(doc);
					} catch(e) {
						var apiresults = {"a":"","n":"","d":"","r":"","l":"--","c":""};
					}
					if (apiresults && apiresults.l)
					{
						wip_storage.set("iplock"+ip, false);
						if (!apiresults.r || apiresults.r==0) apiresults.r = "";
						if (ipcache && ipcache.r && !apiresults.r) apiresults.r = ipcache.r;
						if (!apiresults.a || apiresults.a==0) apiresults.a = "";
						if (!apiresults.n || apiresults.n==0) apiresults.n = "";
						if (!apiresults.c || apiresults.c==0) apiresults.c = "";
						if (!apiresults.d || apiresults.d==0) apiresults.d = "";
						if (apiresults.l=="XX" || apiresults.l=="AP" || apiresults.l=="EU" || apiresults.l=="--") {
							if (apiresults.l=="XX" && apiresults.c)
								apiresults.l = "--"
							wip_storage.set("ip"+ip, {"l":apiresults.l,"a":apiresults.a,"r":apiresults.r,"c":apiresults.c}, 3600); // Land, AS number, RDNS, City and Region code
						}
						else
							wip_storage.set("ip"+ip, {"l":apiresults.l,"a":apiresults.a,"r":apiresults.r,"c":apiresults.c}); // Land, AS number, RDNS, City and Region code
						if (!wip_storage.get("as"+apiresults.a))
							wip_storage.set("as"+apiresults.a, {"n":apiresults.n,"d":apiresults.d}); // AS name, Provider/Datacenter

						if (barRenew)
							wip.showInfo(ip,host);
						if (host) { // check not for myip, ob api langsamer wird
							if ( wip.prefs["nextServerTime"] > 2 * wip.prefs["thisServerTime"] )
							{
								if ( reqTime >= wip.prefs["nextServerTime"] && (endTime-wip.longLastRequestsDate) > 3000 )
								{
									wip.longLastRequestsDate = endTime;
									wip.longLastRequests += 1;
								}
							}
							else if ( reqTime > (2 * wip.prefs["thisServerTime"]) && (endTime-wip.longLastRequestsDate) > 3000 )
							{
								wip.longLastRequestsDate = endTime;
								wip.longLastRequests += 1;
							}
							if ( reqTime < (1.5 * wip.prefs["thisServerTime"]) )
								wip.longLastRequests -= 1;
							if (wip.longLastRequests < 0)
								wip.longLastRequests = 0;
							else if (wip.longLastRequests > 20 && wip.prefs["APIServerFolge"].length > 1)
							{
								wip.preferences.setIntPref(wip.currentAPI+"ResponseTime", -1);
								var newFolge = wip.prefs["APIServerFolge"].substring(1);
								wip.preferences.setCharPref("APIServerFolge",newFolge);
								wip.initPrefs();
								wip.longLastRequests = 0;
							}
						}
					}
				}
			};
			if (wip.currentAPI)
			{
				var startTime = new Date().getTime();
				req.send(null);
				var thisTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
				var currentAPI = wip.currentAPI;
				var event = { notify: function(timer) { wip.isResponseComing(ip,host,currentAPI); } };
				try { thisTimer.initWithCallback(event,(3*wip.prefs["thisServerTime"]+5000),WIP_TYPE_ONE_SHOT); } catch(e) {}
			}
			return true;
		}
		else
		{
			if (!wip.prefs["enablelands"])
				if (!ipcache)
					wip_storage.set("ip"+ip, {"l":"--","a":"","r":"","c":""});
			if (barRenew)
				wip.showInfo(ip,host);
			return null;
		}
	},

	isResponseComing: function(ip,host,currentAPI)
	{
		var ipcache = wip_storage.get("ip"+ip);
		if ((!ipcache || (ipcache.l=="--" && !ipcache.c) ))
		{
			wip_storage.set("iplock"+ip, false);
			if (wip.currentAPI==currentAPI)
			{
				wip.preferences.setIntPref(currentAPI+"ResponseTime", -1);
				var newFolge = wip.prefs["APIServerFolge"].substring(1);
				wip.preferences.setCharPref("APIServerFolge",newFolge);
				wip.initPrefs();
				wip.getMyIP();
			}
			var thisTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
			var event = { notify: function(timer) { wip.showInfoActual(); } };
			try { thisTimer.initWithCallback(event,300,WIP_TYPE_ONE_SHOT); } catch(e) {}
			wip.getLand(ip,host,0);
		}
	},

	myip: function()
	{
		var myip = wip_storage.get("myip");
		if (myip || myip==="")
			return myip;
		var lastmyip = wip_storage.get("myiplast");
		var ipTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
		var event = { notify: function(timer) { wip.changeMyIP(); } };
		try { ipTimer.initWithCallback(event,50,WIP_TYPE_ONE_SHOT); } catch(e) {}
		if (lastmyip)
			return lastmyip;
		return "";
	},
	
	getMyIP: function()
	{
		if (wip_storage.get("myiplock"))
			return;
		wip_storage.set("myiplock", true, 10);
		var req = new XMLHttpRequest();
		req.open("GET", "http://"+wip.currentAPI+".wipmania.com/", true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var doc = req.responseText;
				try
				{
					var myData = doc.split("<br>");
					var myip = myData[0].substring(0,15);
					var myland = myData[1].substring(0,2);
					var myiplast = wip_storage.get("myiplast");
					if (myip && wip.prefs["myiplogenable"] && (!myiplast || myiplast != myip)) {
						var myiplog = wip_storage.get("myiplog");
						var zeit = new Date().toLocaleString();
						if (!myiplog)
							myiplog = new Array();
						if (myiplog.length==0 || myiplog[myiplog.length-1].ip != myip) {
							myiplog.push({"ip":myip,"time":String(zeit)});
							wip_storage.set("myiplog", myiplog);
						}
					}
					if (wip.prefs["updatemyipinterval"] > 0)
						wip_storage.set("myip", myip, wip.prefs["updatemyipinterval"]);
					else
						wip_storage.set("myip", myip);
					wip_storage.set("myiplast", myip);
					var ipcache = wip_storage.get("ip"+myip);
					if (ipcache == null || ipcache.l != myland)
						wip_storage.set("ip"+myip, {"l":myland,"a":"","r":"","c":""});
					if (wip.prefs["lastmyland"]!=myland)
					{
						wip.preferences.setCharPref("lastmyland", myland);
						if (wip.prefs["lastmyland"])
						{
							wip.preferences.setCharPref("APIServerListMD5","");
							wip.prefs["APIServerListMD5"]= "";
							wip.updateAPIFolge();
						}
						wip.prefs["lastmyland"]= myland;
					}
					if (ipcache == null || ipcache.l != myland || wip.prefs["lastmyland"]!=myland)
						wip.getLand(myip,"",0);
				}
				catch(e)
				{
					if (wip.prefs["updatemyipinterval"] > 0)
						wip_storage.set("myip", "", wip.prefs["updatemyipinterval"]);
					else
						wip_storage.set("myip", "");
				}
				wip.setPanelAttributes();
			}
		};
		if (wip.currentAPI)
			req.send(null);
		return true;
	},

	showInfoActual: function()
	{
		var Location = getBrowser().contentDocument.location;
		var host = null;
		if (Location && Location.host && (Location.scheme != "chrome") && (Location.scheme != "file") && (Location.scheme != "about"))
			host = Location.host;
		else if ( Location.scheme == "chrome" || Location.scheme == "file" )
			host = Location;
		var ip = wip.ipResolve(host) ? wip.ipResolve(host) : null;
		wip.showInfo(ip,host);
	},

	showInfo: function(ip,host)
	{
		var ipcache = wip_storage.get("ip"+ip);
		if (ipcache && ipcache.l) {
			var flagimage = ipcache.l;
			var country = ipcache.l;
		}
		else {
			var country = "";
			var flagimage = "--";
			if (ip && (ip=="V6" || ip.indexOf(":")>=0))
				country = "V6";
			else if (!wip.isPublicNetz(ip))
			{
				if (host)
				{
					var PrivateType = wip.getPrivateType(ip);
					var flagimage = (PrivateType=="localhost") ? "20" : "21";
					country = PrivateType ? PrivateType : "localhost";
					if (!ip && wip.proxy_dns_enabled && !PrivateType) {
						country = "--";
						flagimage = "--";
					}
				}
			}
			else if (ip && wip.currentAPI)
			{
				wip.getLand(ip,host,0);
				var thisTimer = WIPCc["@mozilla.org/timer;1"].createInstance(WIPCi.nsITimer);
				var event = { notify: function(timer) { wip.showInfoActual(); } };
				try { thisTimer.initWithCallback(event,300,WIP_TYPE_ONE_SHOT); } catch(e) {}
				return;
			}
			else
				country = "--";
		}

		var src = "chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/"+flagimage+".png";
		var stringbundle = document.getElementById("worldip-tooltiplabels");
		var panel = document.getElementById("worldip-statusbar");
		var panel_flag = document.getElementById("worldip-statusbar-flag");
		var panel_land = document.getElementById("worldip-statusbar-land");
		var panel_landcode = document.getElementById("worldip-statusbar-landcode");
		var panel_city = document.getElementById("worldip-statusbar-city");
		var panel_ip = document.getElementById("worldip-statusbar-ip");

		panel_flag.setAttribute("src", src);
		panel_landcode.setAttribute("value", (ipcache && ipcache.l) ? ipcache.l : "");
		if (country) {
			if (country=="--" && wip.proxy_dns_enabled)
				panel_land.setAttribute("value", "DNS "+stringbundle.getString(country)+" (SOCKSv5)");
			else
				panel_land.setAttribute("value", stringbundle.getString(country));
		} else
			panel_land.setAttribute("value", "");
		if (ipcache && ipcache.c)
			panel_city.setAttribute("value", ipcache.c);
		else
			panel_city.setAttribute("value", "");

		var showip = ip;
		var d = wip_storage.get("dns"+host);
		if (d && d.length>1)
			showip = showip + " (+" + (d.length-1) + ")"
		panel_ip.setAttribute("value", showip);
		wip.setPanelAttributes();
		return true;
	},

	setPositon: function()
	{
		switch (wip.prefs["position"])
		{
			case "s": //statusbar
				try
				{	// this function is originally from flagfox extension. Rewrited for compatibility with add-on "Statusable"
					var statusbar = document.getElementById("status-bar");
					var worldipbar = document.getElementById("worldip-statusbar");
					switch (wip.prefs["toolbarposition"])
					{
						case "left":
							var el = statusbar.firstChild;
							break;
						case "center":
							var el = document.getElementById("statusbar-progresspanel").nextSibling;
							if (!el)
								el = document.getElementById("download-monitor").nextSibling;
							break;
						case "right":
							var el = statusbar.lastChild.nextSibling;
					}
					statusbar.insertBefore(worldipbar,el);
				} catch(e) {}
				break;
			case "a": //addressbar
				try
				{
					var worldipbar = document.getElementById("worldip-addressbar");
					switch (wip.prefs["toolbarposition"])
					{
						case "left":
							var addressbar = document.getElementById("urlbar");
							var el = addressbar.firstChild.nextSibling;
							break;
						case "center":
							var addressbar = document.getElementById("urlbar-icons");
							var el = addressbar.firstChild;
							break;
						case "right":
							var addressbar = document.getElementById("urlbar-icons");
							var el = addressbar.lastChild.nextSibling;
					}
					addressbar.insertBefore(worldipbar, el);
				} catch(e) {}
				break;
		}
	},

	setPanelAttributesStatusbar: function(ip)
	{
		var panel_flag = document.getElementById("worldip-statusbar-flag");
		var panel_land = document.getElementById("worldip-statusbar-land");
		var panel_landcode = document.getElementById("worldip-statusbar-landcode");
		var panel_city = document.getElementById("worldip-statusbar-city");
		var panel_ip = document.getElementById("worldip-statusbar-ip");
		var panel_datacenter = document.getElementById("worldip-statusbar-datacenter");
		var panel_asnumber = document.getElementById("worldip-statusbar-asnumber");
		var panel_asname = document.getElementById("worldip-statusbar-asname");
		var panel_rdns = document.getElementById("worldip-statusbar-rdns");
		var panel_myip = document.getElementById("worldip-statusbar-myip");
		var panel_mydatacenter = document.getElementById("worldip-statusbar-mydatacenter");
		var panel_myasnumber = document.getElementById("worldip-statusbar-myasnumber");
		var panel_myasname = document.getElementById("worldip-statusbar-myasname");
		var panel_myrdns = document.getElementById("worldip-statusbar-myrdns");

		panel_flag.setAttribute("hidden", wip.prefs["showflag"] ? "false" : "true");
		
		panel_landcode.setAttribute("hidden", wip.prefs["showlandcode"] ? "false" : "true");
		if (panel_landcode.getAttribute("value") == "")
			panel_landcode.setAttribute("hidden", "true");
		panel_landcode.setAttribute("style", "color:"+wip.prefs["showlandcodecolor"]+";");
		
		panel_land.setAttribute("hidden", wip.prefs["showland"] ? "false" : "true");
		if (panel_land.getAttribute("value") == "")
			panel_land.setAttribute("hidden", "true");
		panel_land.setAttribute("style", "color:"+wip.prefs["showlandcolor"]+";");

		panel_ip.setAttribute("hidden", wip.prefs["showip"] ? "false" : "true");
		if (panel_ip.getAttribute("value") == "")
			panel_ip.setAttribute("hidden", "true");
		panel_ip.setAttribute("style", "color:"+wip.prefs["showipcolor"]+";");

		var myip = wip.myip();
		panel_myip.setAttribute("value", myip ? myip : "");
		panel_myip.setAttribute("style", "color:"+wip.prefs["showmyipcolor"]+";");
		panel_myip.setAttribute("hidden", (wip.prefs["showmyip"] && myip && wip.prefs["enablemyip"]) ? "false" : "true");

		var myipcache = wip_storage.get("ip"+myip);
		var myascache = "";
		if (myipcache && myipcache.a)
			myascache = wip_storage.get("as"+myipcache.a);
		if (myascache) {
			var mydatacenter = myascache.d;
			if (mydatacenter.length>40)
				mydatacenter = mydatacenter.substring(0,40)+"…";
			panel_mydatacenter.setAttribute("value", mydatacenter);
			panel_mydatacenter.setAttribute("style", "color:"+wip.prefs["showmydatacentercolor"]+";");
			panel_mydatacenter.setAttribute("hidden", (myascache.d && wip.prefs["showmydatacenter"] && myip && wip.prefs["enablemyip"]) ? "false" : "true");

			panel_myasnumber.setAttribute("value", "AS"+myipcache.a);
			panel_myasnumber.setAttribute("style", "color:"+wip.prefs["showmyasnumbercolor"]+";");
			panel_myasnumber.setAttribute("hidden", (myipcache.a && wip.prefs["showmyasnumber"] && myip && wip.prefs["enablemyip"]) ? "false" : "true");

			panel_myasname.setAttribute("value", myascache.n);
			panel_myasname.setAttribute("style", "color:"+wip.prefs["showmyasnamecolor"]+";");
			panel_myasname.setAttribute("hidden", (myascache.n && wip.prefs["showmyasname"] && myip && wip.prefs["enablemyip"]) ? "false" : "true");
		} else {
			panel_mydatacenter.setAttribute("hidden", "true");
			panel_myasnumber.setAttribute("hidden", "true");
			panel_myasname.setAttribute("hidden", "true");
		}

		if (myipcache && myipcache.r) {
			panel_myrdns.setAttribute("value", myipcache.r);
			panel_myrdns.setAttribute("style", "color:"+wip.prefs["showmyrdnscolor"]+";");
			panel_myrdns.setAttribute("hidden", (myipcache.r && wip.prefs["showmyrdns"] && myip && wip.prefs["enablemyip"]) ? "false" : "true");
		}
		else
			panel_myrdns.setAttribute("hidden", "true");

		var ipcache = wip_storage.get("ip"+ip);
		var ascache = "";
		if (ipcache && ipcache.a)
			ascache = wip_storage.get("as"+ipcache.a);
		if (ascache) {
			var datacenter = ascache.d;
			if (datacenter.length>40)
				datacenter = datacenter.substring(0,40)+"…";
			panel_datacenter.setAttribute("value", datacenter);
			panel_datacenter.setAttribute("style", "color:"+wip.prefs["showdatacentercolor"]+";");
			panel_datacenter.setAttribute("hidden", (ascache.d && wip.prefs["showdatacenter"] && ip) ? "false" : "true");

			panel_asname.setAttribute("value", ascache.n);
			panel_asname.setAttribute("style", "color:"+wip.prefs["showasnamecolor"]+";");
			panel_asname.setAttribute("hidden", (ascache.n && wip.prefs["showasname"] && ip) ? "false" : "true");
		} else {
			panel_datacenter.setAttribute("hidden", "true");
			panel_asname.setAttribute("hidden", "true");
		}
		if (ipcache) {
			panel_asnumber.setAttribute("value", ipcache.a);
			panel_asnumber.setAttribute("style", "color:"+wip.prefs["showasnumbercolor"]+";");
			panel_asnumber.setAttribute("hidden", (ipcache.a && wip.prefs["showasnumber"] && ip) ? "false" : "true");

			panel_city.setAttribute("style", "color:"+wip.prefs["showcitycolor"]+";");
			panel_city.setAttribute("hidden", (ipcache.c && wip.prefs["showcity"] && ip) ? "false" : "true");

			panel_rdns.setAttribute("value", ipcache.r);
			panel_rdns.setAttribute("style", "color:"+wip.prefs["showrdnscolor"]+";");
			panel_rdns.setAttribute("hidden", (ipcache.r && wip.prefs["showrdns"] && ip) ? "false" : "true");
		} else {
			panel_asnumber.setAttribute("hidden", "true");
			panel_city.setAttribute("hidden", "true");
			panel_rdns.setAttribute("hidden", "true");
		}
	},

	setPanelAttributesAddressbar: function(ip)
	{
		var statusbar_flag = document.getElementById("worldip-statusbar-flag");
		var statusbar = statusbar_flag.getAttribute("src");
		var addressbar_flag = document.getElementById("worldip-addressbar-flag");
		addressbar_flag.setAttribute("src", statusbar);
	},

	setPanelAttributes: function()
	{
		var ip = wip.getCurrentIP();
		var ip_v4 = (ip.indexOf(":")>=0) ? false : true;
		switch (wip.prefs["position"])
		{
			case "s": //statusbar
				var statusbar_panel = document.getElementById("worldip-statusbar");
				statusbar_panel.setAttribute("hidden", "false");
				var addressbar_panel = document.getElementById("worldip-addressbar");
				addressbar_panel.setAttribute("hidden", "true");
				wip.setPanelAttributesStatusbar(ip);
				break;
			case "a": //addressbar
				var statusbar_panel = document.getElementById("worldip-statusbar");
				statusbar_panel.setAttribute("hidden", "true");
				var addressbar_panel = document.getElementById("worldip-addressbar");
				addressbar_panel.setAttribute("hidden", "false");
				wip.setPanelAttributesAddressbar(ip);
		}
		var ipcache = wip_storage.get("ip"+ip);
		var ascache = "";
		if (ipcache && ipcache.a)
			ascache = wip_storage.get("as"+ipcache.a);
		else if (ipcache && !ipcache.l)
			ipcache.l = "--";
		// change variables in menu
		var ipmenu = document.getElementById("worldip-ip");
			ipmenu.setAttribute("label", ip ? (wip.worldip_ip_leer+ip) : wip.worldip_ip_leer);
			ipmenu.setAttribute("disabled", ip ? "false" : "true");
			if (ipcache && ipcache.l)
				ipmenu.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/"+ipcache.l+".png);");
			else
				ipmenu.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/--.png);");
		var whoisipmenu = document.getElementById("worldip-whoisip");
			whoisipmenu.setAttribute("label", ip ? ("Whois "+ip+"…") : "Whois");
			whoisipmenu.setAttribute("disabled", wip.isPublicNetz(ip) ? "false" : "true");
		var rdnsipmenu = document.getElementById("worldip-getrdns");
			rdnsipmenu.setAttribute("label", ip ? (wip.worldip_rdns_leer+ip) : "RDNS");
			if (ipcache && (ipcache.r && ipcache.r!=0))
				rdnsipmenu.setAttribute("disabled", "true");
			else
				rdnsipmenu.setAttribute("disabled", wip.isPublicNetz(ip) ? "false" : "true");
		var pingipmenu = document.getElementById("worldip-ping-ip-menu");
			pingipmenu.setAttribute("disabled", (wip.isPublicNetz(ip) && ip_v4) ? "false" : "true");
		var traceipmenu = document.getElementById("worldip-trace-ip-menu");
			traceipmenu.setAttribute("disabled", (wip.isPublicNetz(ip) && ip_v4) ? "false" : "true");
		var copyip = document.getElementById("worldip-copyip");
			copyip.setAttribute("disabled", ip ? "false" : "true");
		var showlandinfo = document.getElementById("worldip-showlandinfo");
			showlandinfo.setAttribute("disabled", (wip.isPublicNetz(ip) && ip && ip_v4 && ipcache && ipcache.l && (ipcache.l!="--" || (ipcache.l=="--" && ipcache.c) ) && wip.currentAPI) ? "false" : "true");
		var reportbadland = document.getElementById("worldip-reportbadland");
			reportbadland.setAttribute("disabled", (wip.isPublicNetz(ip) && ip && ip_v4 && ipcache && ipcache.l && (ipcache.l!="--" || (ipcache.l=="--" && ipcache.c) ) && wip.currentAPI) ? "false" : "true");
		var changehostip = document.getElementById("worldip-changehostip");
			changehostip.setAttribute("disabled", (wip.isPublicNetz(ip) && ip) ? "false" : "true");

		var myiplogmenu = document.getElementById("worldip-myiplog");
			if (wip.prefs["myiplogenable"])
				myiplogmenu.setAttribute("disabled", "false");
			else
				myiplogmenu.setAttribute("disabled", "true");
			
		var getland = document.getElementById("worldip-getland");
		var getipland = document.getElementById("worldip-getipland");
			if (wip.proxy_dns_enabled && !wip.prefs["proxy_dns_uselocal"]) {
				getipland.setAttribute("disabled", (!ip || (ipcache && ((!ipcache.l && !wip.prefs["enablelands"]) || (ipcache.l=="--" && !ipcache.c) ) && wip.isPublicNetz(ip) && wip.currentAPI && ip_v4)) ? "false" : "true");
				getland.setAttribute("disabled", "true");
			} else {
				getland.setAttribute("disabled", ( ipcache && ((!ipcache.l && !wip.prefs["enablelands"]) || (ipcache.l=="--" && !ipcache.c) ) && wip.isPublicNetz(ip) && wip.currentAPI && ip_v4) ? "false" : "true");
				getipland.setAttribute("disabled", "true");
			}

		var myip = wip.myip();
		var worldipmyip = document.getElementById("worldip-myip");
			worldipmyip.setAttribute("label", (wip.prefs["enablemyip"]) ? (wip.worldip_myip_leer+myip) : wip.worldip_myip_leer);
			worldipmyip.setAttribute("disabled", (wip.prefs["enablemyip"]) ? "false" : "true");
			if (myip && wip_storage.get("ip"+myip).l)
				worldipmyip.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/"+wip_storage.get("ip"+myip).l+".png);");
			else
				worldipmyip.setAttribute("style", "list-style-image: url(chrome://worldip/skin/flags"+wip.prefs["flagset"]+"/--.png);");
		var whoismyip = document.getElementById("worldip-whoismyip");
			whoismyip.setAttribute("label", (wip.prefs["enablemyip"]) ? ("Whois "+myip+"…") : "Whois");
			whoismyip.setAttribute("disabled", (wip.prefs["enablemyip"]) ? "false" : "true");
		var rdnsmyipmenu = document.getElementById("worldip-getmyrdns");
			rdnsmyipmenu.setAttribute("label", ip ? (wip.worldip_rdns_leer+myip) : "RDNS");
			var myipcache = wip_storage.get("ip"+myip);
			if (myipcache && (myipcache.r && myipcache.r!=0))
				rdnsmyipmenu.setAttribute("disabled", "true");
			else
				rdnsmyipmenu.setAttribute("disabled", wip.isPublicNetz(myip) ? "false" : "true");
		var enablemyip = document.getElementById("worldip-enablemyip");
			enablemyip.setAttribute("checked", (wip.prefs["enablemyip"]) ? "true" : "false");
		var changemyip = document.getElementById("worldip-changemyip");
			changemyip.setAttribute("disabled", (wip.prefs["enablemyip"] && wip.currentAPI) ? "false" : "true");
		var copymyip = document.getElementById("worldip-copymyip");
			copymyip.setAttribute("disabled", (wip.prefs["enablemyip"] && wip.currentAPI) ? "false" : "true");
		var reportbadmyland = document.getElementById("worldip-reportbadmyland");
			reportbadmyland.setAttribute("disabled", (wip.prefs["enablemyip"] && wip.currentAPI) ? "false" : "true");
	},

	showMessage: function(message,clicked)
	{
		if (!message) return;
		var alertsService = WIPCc["@mozilla.org/alerts-service;1"].getService(WIPCi.nsIAlertsService);
		alertsService.showAlertNotification("chrome://worldip/skin/logo.png","WorldIP", message,clicked, "", null);
	},

	hideTooltip: function()
	{
		document.getElementById( "worldip-statusbar" ).setAttribute( "tooltip", "" );
	},

	showTooltip: function()
	{
		document.getElementById( "worldip-statusbar" ).setAttribute( "tooltip", "worldip-tooltip" );
		document.getElementById( "worldip-tooltip" ).setAttribute( "noautohide", "true" );
	},

	renewTooltip: function()
	{
		if (!wip.currentIP)
			wip.currentIP = wip.getCurrentIP();
		if (!wip.currentUri)
			wip.currentUri = getBrowser().contentDocument.location;
		var ipcache = wip_storage.get("ip"+wip.currentIP);
		var countryCode = (ipcache && ipcache.l) ? ipcache.l : "";
		var tooltipdata = {};
		var stringbundle = document.getElementById("worldip-tooltiplabels");
		var PrivateType = wip.getPrivateType(wip.currentIP);
		if (wip.currentUri == "about:blank")
		{
			tooltipdata["hostname"] = "about:blank";
			tooltipdata["country"] = "localhost";
		}
		else if (wip.currentUri.host.indexOf(":")>0)
		{
			tooltipdata["hostname"] = wip.currentUri.host;
			tooltipdata["country"] = "V6";
		}
		else if (!countryCode)
		{
			try {
				tooltipdata["hostname"] = (wip.currentUri == "about:config") ? "about:config" : wip.currentUri.host;
			} catch(e) {
				tooltipdata["hostname"] = wip.currentUri;
			}
			if (wip.currentIP) {
				tooltipdata["hostip"] = wip.currentIP;
				var d = wip_storage.get("dns"+wip.currentUri.host);
				tooltipdata["hostiprest"] = (d && d.length>1) ? d.slice(1) : "";
			}
			if (wip.currentAPI)
				tooltipdata["country"] = PrivateType ? PrivateType : "localhost";
			if (wip.proxy_dns_enabled)
				tooltipdata["country"] = "";
		}
		else
		{
			tooltipdata["hostname"] = wip.currentUri.host;
			tooltipdata["hostip"] = wip.currentIP;
			var d = wip_storage.get("dns"+wip.currentUri.host);
			tooltipdata["hostiprest"] = (d && d.length>1) ? d.slice(1) : "";
			tooltipdata["country"] = countryCode;
			tooltipdata["countrycode"] = countryCode;
		}
		if (ipcache)
		{
			if (ipcache.c) tooltipdata["city"] = ipcache.c; else tooltipdata["city"] = "";
			if (ipcache.r) tooltipdata["rdns"] = ipcache.r; else tooltipdata["rdns"] = "";
		}
		else
		{
			tooltipdata["city"] = "";
			tooltipdata["rdns"] = "";
		}
		var ascache = "";
		if (ipcache && ipcache.a)
			ascache = wip_storage.get("as"+ipcache.a);
		if (ascache)
		{
			if (ascache.d) tooltipdata["datacenter"] = ascache.d; else tooltipdata["datacenter"] = "";
			tooltipdata["asnumber"] = ipcache.a;
			if (ascache.n) tooltipdata["asname"] = ascache.n; else tooltipdata["asname"] = "";
		}
		else
		{
			tooltipdata["datacenter"] = "";
			tooltipdata["asnumber"] = "";
			tooltipdata["asname"] = "";
		}
		if (wip.currentIP && !PrivateType)
		{
			var ianastatus = wip.getIANAStatus(wip.currentIP);
			if (ianastatus[2]=="ALLOCATED")
				tooltipdata["ianastatus"] = ianastatus[0];
			else if (ianastatus[2]=="UNALLOCATED")
				tooltipdata["ianastatus"] = ianastatus[2];
			else
				tooltipdata["ianastatus"] = ianastatus[0]+" ("+ianastatus[2]+")";
		}
		var myip = wip.myip();
		if (wip.prefs["enablemyip"] && myip)
		{
			tooltipdata["myip"] = myip;
			tooltipdata["myland"] = wip_storage.get("ip"+myip).l;
			tooltipdata["mylandcode"] = wip_storage.get("ip"+myip).l;
		}

		var myipcache = wip_storage.get("ip"+myip);
		var myascache = "";
		if (myipcache && myipcache.a)
			myascache = wip_storage.get("as"+myipcache.a);
		if (myipcache && myipcache.r) tooltipdata["myrdns"] = myipcache.r; else tooltipdata["myrdns"] = "";
		if (myascache)
		{
			if (myascache.d) tooltipdata["mydatacenter"] = myascache.d; else tooltipdata["mydatacenter"] = "";
			tooltipdata["myasnumber"] = myipcache.a;
			if (myascache.n) tooltipdata["myasname"] = myascache.n; else tooltipdata["myasname"] = "";
		}
		else
		{
			tooltipdata["mydatacenter"] = "";
			tooltipdata["myasnumber"] = "";
			tooltipdata["myasname"] = "";
		}

		var tooltip = document.getElementById("worldip-tooltipdata");

		while (tooltip.lastChild)
			tooltip.removeChild(tooltip.lastChild);
		for (var element in tooltipdata)
		{
			if (element=="myip" || element=="hostname")
				tooltip.appendChild(document.createElement("menuseparator"));
			if (tooltipdata[element]) {
				if (element=="hostiprest") {
					for ( i in tooltipdata[element] ) {
						var row = document.createElement("row");
						var label = document.createElement("label");
						label.setAttribute("value", "");
						var value = document.createElement("label");
						value.appendChild(document.createTextNode(tooltipdata[element][i]));
						value.style.whiteSpace = "pre";
						row.appendChild(label);
						row.appendChild(value);
						tooltip.appendChild(row);
					}
				} else {
					var row = document.createElement("row");
					var labelstring = stringbundle.getString(element) + ":";
					var label = document.createElement("label");
					label.setAttribute("value", labelstring);
					label.style.fontWeight = "bold";
					var value = document.createElement("label");
					if ((element == "country") || ((element == "myland") && tooltipdata["myland"]))
						var valuedata = stringbundle.getString(tooltipdata[element]);
					else
						var valuedata = tooltipdata[element];
					value.appendChild(document.createTextNode(valuedata));
					value.style.whiteSpace = "pre";
					row.appendChild(label);
					row.appendChild(value);
					tooltip.appendChild(row);
				}
			}
		}
		document.getElementById( "worldip-statusbar" ).setAttribute( "tooltip", "worldip-tooltip" );
		document.getElementById( "worldip-tooltip" ).setAttribute( "noautohide", "true" );
	},

	showAbout: function()
	{
		window.open("chrome://worldip/content/about.xul", "worldip", "centerscreen,chrome");
	},

	MyIPLogTabl: function(myiplogpanel)
	{
		if (!wip.prefs["myiplogenable"])
			return;
		var myiplog = wip_storage.get("myiplog");
		if (!myiplog)
			return "";
		var all_data = "";
		for (l in myiplog) {
			var ipcache = wip_storage.get("ip"+myiplog[l].ip);
			if (ipcache == null || !ipcache.l)
				ipcache = {l:"", a:""};
			if (ipcache.a)
				var ascache = wip_storage.get("as"+ipcache.a);
			else
				var ascache = {n:"",d:""};
			var data = myiplog[l].ip + "	"+ ipcache.l + "	" + myiplog[l].time + "	" + " AS" + ipcache.a + "	" + ascache.d;
			if (data.length>100)
				data = data.substring(0,100)+"…";
			all_data += data+"\n";
		}
		myiplogpanel.setAttribute( "value", all_data );
	},
	
	showMyIPLog: function()
	{
		var w = window.open("chrome://worldip/content/myip.xul", "worldiplog", "centerscreen,chrome,resizable=yes,scrollbars=yes");
		wip.setTimer(function() {
			var myiplogpanel = w.document.getElementById("myiplogpanel-text");
			wip.MyIPLogTabl(myiplogpanel);
		}, 50, WIP_TYPE_ONE_SHOT);
	},

	showOptions: function()
	{
		var optwin = WIPCc['@mozilla.org/appshell/window-mediator;1'].getService(WIPCi.nsIWindowMediator).getMostRecentWindow("WorldIP:Options");
		if (optwin)
			optwin.focus();
		else
			openDialog("chrome://worldip/content/options.xul", "", "centerscreen,chrome,titlebar,toolbar");
	},

	goHome: function(url)
	{
		var browser = document.getElementById("content");
		var tab = browser.addTab("http://www.wipmania.com/"+url);
		browser.selectedTab = tab;
	},

	getCurrentIP: function()
	{
		var ipe = document.getElementById("worldip-statusbar-ip");
		var ipline = ipe.getAttribute("value");
		return ipline.split(" ")[0];
	},

	copyIP: function(ip)
	{
		const copytoclip = WIPCc["@mozilla.org/widget/clipboardhelper;1"]
			.getService(WIPCi.nsIClipboardHelper);
		copytoclip.copyString(ip);
	},


	runProgram: function(path, arg)
	{
		var args = arg ? arg.split(/\s+/) : [];
		var nsILocalFile = WIPCc["@mozilla.org/file/local;1"].getService(WIPCi.nsILocalFile);
		var nsIProcess = WIPCc["@mozilla.org/process/util;1"].createInstance(WIPCi.nsIProcess);
		nsILocalFile.initWithPath(path);
		nsIProcess.init(nsILocalFile);
		nsIProcess.run(false, args, args.length);
	},

	runLocalPing: function(ip)
	{
		var pattern = /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/;
		if (!pattern.test(ip)) return;
		if (WIP_OS=="WINNT")
			wip.runProgram("C:\\WINDOWS\\system32\\cmd.exe","/C start cmd.exe /K ping -n 5 "+ip);
		else //Linux, Darwin, etc
			wip.runProgram("/usr/bin/xterm","-hold -e ping -c 5 "+ip);
	},

	runLocalTrace: function(ip)
	{
		var pattern = /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/;
		if (!pattern.test(ip)) return;
		if (WIP_OS=="WINNT")
			wip.runProgram("C:\\WINDOWS\\system32\\cmd.exe","/C start cmd.exe /K tracert "+ip);
		else
			wip.runProgram("/usr/bin/xterm","-hold -e traceroute "+ip);
	},

	showLandInfo: function(ip)
	{
		var land = (wip_storage.get("ip"+ip) && wip_storage.get("ip"+ip).l) ? wip_storage.get("ip"+ip).l : "";
		if (ip && land && wip.isPublicNetz(ip))
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://www.wipmania.com/map/"+land+"/");
			browser.selectedTab = tab;
		}
	},

	reportBadLand: function(ip)
	{
		var land = wip_storage.get("ip"+ip) ? wip_storage.get("ip"+ip).l : "";
		if (ip && land && wip.isPublicNetz(ip) )
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://www.wipmania.com/base/ip/?ip="+ip+"&ff");
			browser.selectedTab = tab;
		}
	},

	goHomeTrace: function(ip,city)
	{
		if (ip && wip.isPublicNetz(ip))
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://www.wipmania.com/tracefrom/"+ip+"/?c="+city+"&ff=");
			browser.selectedTab = tab;
		}
	},

	goHomePing: function(ip,city)
	{
		if (ip && wip.isPublicNetz(ip))
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://www.wipmania.com/pingfrom/"+ip+"/?c="+city+"&ff=");
			browser.selectedTab = tab;
		}
	},

	goHomeWhois: function(ip)
	{
		if (ip && wip.isPublicNetz(ip))
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://www.wipmania.com/whois/"+ip+"/?ff");
			browser.selectedTab = tab;
		}
	},

	goProvider: function(provurl)
	{
		if (provurl)
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab(provurl);
			browser.selectedTab = tab;
		}
	},

	enableMyIP: function(from_menu)
	{
		if (wip.prefs["enablemyip"])
		{
			if (from_menu)
				wip.preferences.setBoolPref("enablemyip",false);
			wip.prefs["enablemyip"]= wip.preferences.getBoolPref("enablemyip");
			wip_storage.set("myip", "");
			wip_storage.set("myiplast", "");
			wip.setPanelAttributes();
		}
		else
		{
			if (from_menu)
				wip.preferences.setBoolPref("enablemyip",true);
			wip.prefs["enablemyip"]= wip.preferences.getBoolPref("enablemyip");
			wip.changeMyIP();
		}
		if (!wip.prefs["enablemyip"])
			try { wip.MyIPTimer.cancel(); } catch(e) {}
		document.getElementById( "worldip-statusbar" ).setAttribute( "tooltip", "worldip-tooltip" );
	},

	changeMyIP: function()
	{
		if (wip.prefs["enablemyip"]) {
			if (wip.prefs["updatemyipinterval"] > 0)
			{
				try { wip.MyIPTimer.cancel(); } catch(e) {}
				var event = { notify: function(timer) { wip.getMyIP(); } };
				try { wip.MyIPTimer.initWithCallback(event, (wip.prefs["updatemyipinterval"]*1000), WIP_TYPE_REPEATING_SLACK); } catch(e) {}
			}
			wip.getMyIP();
		}
		else
		{
			try { wip.MyIPTimer.cancel(); } catch(e) {}
			wip_storage.set("myip", "");
			wip_storage.set("myiplast", "");
			wip.setPanelAttributes();
		}
		document.getElementById( "worldip-statusbar" ).setAttribute( "tooltip", "worldip-tooltip" );
	},

	changeHostIP: function()
	{
		if (wip.currentUri.host)
		{
			wip_storage.delete("dns"+wip.currentUri.host);
			var ip = wip.ipResolve(wip.currentUri.host);
			if (ip)
				wip.getLand(ip,wip.currentUri.host,1);
		}
	},

	getRDNS: function(ip)
	{
		if (!ip || !wip.isPublicNetz(ip))
			return false;
		var ipcache = wip_storage.get("ip"+ip);
		if (ipcache && ipcache.r && ipcache.r!=0)
			return false;
		var req = new XMLHttpRequest();
		req.open("GET", "http://api.wipmania.com/rdns/?ip="+ip, true);
		req.setRequestHeader("Accept", "text/html,text/plain");
		req.setRequestHeader("Accept-Encoding", null);
		req.setRequestHeader("Keep-Alive", null);
		req.setRequestHeader("Accept-Charset", null);
		req.setRequestHeader("Connection", "close");
		req.setRequestHeader("Typ", "ffplugin2");
		req.setRequestHeader("Ver", WIP_VERSION);
		req.onreadystatechange = function()
		{
			if (req.readyState == 4)
			{
				var doc = req.responseText;
				var pattern = /^[-0-9a-z\.]+$/i;
				if (doc && !pattern.test(doc)) doc = 0;
				if (!doc && doc==="") doc = "not found";
				if (ipcache)
					ipcache.r = doc;
				else
					wip_storage.set("ip"+ip, {"l":"--","a":"","r":doc,"c":""});
				wip.setPanelAttributes();
			}
		};
		req.send(null);
	},

	showASonHE: function(ip) {
		var ipcache = wip_storage.get("ip"+ip);
		if (!ipcache || !ipcache.a)
			return;
		if (ip && wip.isPublicNetz(ip))
		{
			var browser = document.getElementById("content");
			var tab = browser.addTab("http://bgp.he.net/AS"+ipcache.a);
			browser.selectedTab = tab;
		}
	},

	onBarClick: function(e)
	{
		if (e && (e.button == 0) && (!e.ctrlKey && !e.shiftKey))
		{
			var action = wip.prefs["clickaction"];
			switch(action)
			{
				case "no":
					break;
				case "land":
					wip.showLandInfo(wip.getCurrentIP());
					break;
				case "rdns":
					wip.getRDNS(wip.getCurrentIP());
					break;
				case "myrdns":
					wip.getRDNS(wip.myip());
					break;
				case "copy":
					wip.copyIP(wip.getCurrentIP());
					break;
				case "newip":
					wip.changeMyIP();
					break;
				case "wrongland":
					wip.reportBadLand(wip.getCurrentIP());
					break;
				case "getipland":
					wip.ipRemoteResolve();
					break;
				case "getland":
					wip.getDisabledLand();
					break;
				case "asonhe":
					wip.showASonHE(wip.getCurrentIP());
					break;
			}
		}
	},
}
